@@ -1,3 +1,8 @@
+#[cfg(feature = "actix3-validator")]
+extern crate actix_web_validator2 as actix_web_validator;
+#[cfg(feature = "actix4-validator")]
+extern crate actix_web_validator3 as actix_web_validator;
+
 #[cfg(feature = "actix-multipart")]
 use super::schema::TypedData;
 use super::{
@@ -7,17 +12,27 @@ use super::{
     },
     schema::{Apiv2Errors, Apiv2Operation, Apiv2Schema},
 };
+#[cfg(not(feature = "actix4"))]
 use crate::util::{ready, Ready};
-#[cfg(feature = "actix3")]
+#[cfg(any(feature = "actix3", feature = "actix4"))]
 use actix_web::web::ReqData;
+#[cfg(not(feature = "actix4"))]
+use actix_web::Error;
+#[cfg(feature = "actix4")]
+use actix_web::{body::BoxBody, ResponseError};
 use actix_web::{
     http::StatusCode,
     web::{Bytes, Data, Form, Json, Path, Payload, Query},
-    Error, HttpRequest, HttpResponse, Responder,
+    HttpRequest, HttpResponse, Responder,
 };
 
 use pin_project::pin_project;
 
+#[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+use actix_web_validator::{
+    Json as ValidatedJson, Path as ValidatedPath, QsQuery as ValidatedQsQuery,
+    Query as ValidatedQuery,
+};
 use serde::Serialize;
 #[cfg(feature = "serde_qs")]
 use serde_qs::actix::QsQuery;
@@ -154,10 +169,10 @@ where
 impl<T> Apiv2Schema for Data<T> {}
 #[cfg(not(feature = "nightly"))]
 impl<T> OperationModifier for Data<T> {}
-#[cfg(feature = "actix3")]
+#[cfg(any(feature = "actix3", feature = "actix4"))]
 impl<T: std::clone::Clone> Apiv2Schema for ReqData<T> {}
 #[cfg(not(feature = "nightly"))]
-#[cfg(feature = "actix3")]
+#[cfg(any(feature = "actix3", feature = "actix4"))]
 impl<T: std::clone::Clone> OperationModifier for ReqData<T> {}
 
 macro_rules! impl_empty({ $($ty:ty),+ } => {
@@ -168,6 +183,53 @@ macro_rules! impl_empty({ $($ty:ty),+ } => {
     )+
 });
 
+#[cfg(feature = "actix4")]
+/// Workaround for possibility to directly return HttpResponse from closure handler.
+///
+/// This is needed after actix removed `impl Future` from `HttpResponse`:
+/// <https://github.com/actix/actix-web/pull/2601>
+///
+/// Example:
+//////
+/// ```ignore
+/// .route(web::get().to(||
+///     async move {
+///         paperclip::actix::HttpResponseWrapper(
+///             HttpResponse::Ok().body("Hi there!")
+///         )
+///     }
+/// ))
+/// ```
+pub struct HttpResponseWrapper(pub HttpResponse);
+
+#[cfg(feature = "actix4")]
+impl Responder for HttpResponseWrapper {
+    type Body = <HttpResponse as Responder>::Body;
+
+    fn respond_to(self, req: &HttpRequest) -> HttpResponse<Self::Body> {
+        self.0.respond_to(req)
+    }
+}
+
+#[cfg(feature = "actix4")]
+impl<F> Apiv2Operation for F
+where
+    F: Future<Output = HttpResponseWrapper>,
+{
+    fn operation() -> DefaultOperationRaw {
+        Default::default()
+    }
+
+    fn security_definitions() -> BTreeMap<String, SecurityScheme> {
+        Default::default()
+    }
+
+    fn definitions() -> BTreeMap<String, DefaultSchemaRaw> {
+        Default::default()
+    }
+}
+
+#[cfg(not(feature = "actix4"))]
 impl Apiv2Operation for HttpResponse {
     fn operation() -> DefaultOperationRaw {
         Default::default()
@@ -229,7 +291,9 @@ impl<T: chrono::TimeZone> OperationModifier for chrono::DateTime<T> {}
 
 #[cfg(feature = "nightly")]
 impl<T> Apiv2Schema for Json<T> {
-    default const NAME: Option<&'static str> = None;
+    default fn name() -> Option<String> {
+        None
+    }
 
     default fn raw_schema() -> DefaultSchemaRaw {
         Default::default()
@@ -238,7 +302,9 @@ impl<T> Apiv2Schema for Json<T> {
 
 /// JSON needs specialization because it updates the global definitions.
 impl<T: Apiv2Schema> Apiv2Schema for Json<T> {
-    const NAME: Option<&'static str> = T::NAME;
+    fn name() -> Option<String> {
+        T::name()
+    }
 
     fn raw_schema() -> DefaultSchemaRaw {
         T::raw_schema()
@@ -281,6 +347,45 @@ where
     }
 }
 
+#[cfg(all(
+    any(feature = "actix4-validator", feature = "actix3-validator"),
+    feature = "nightly"
+))]
+impl<T> Apiv2Schema for ValidatedJson<T> {
+    fn name() -> Option<String> {
+        None
+    }
+
+    default fn raw_schema() -> DefaultSchemaRaw {
+        Default::default()
+    }
+}
+
+#[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+impl<T: Apiv2Schema> Apiv2Schema for ValidatedJson<T> {
+    fn name() -> Option<String> {
+        T::name()
+    }
+
+    fn raw_schema() -> DefaultSchemaRaw {
+        T::raw_schema()
+    }
+}
+
+#[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+impl<T> OperationModifier for ValidatedJson<T>
+where
+    T: Apiv2Schema,
+{
+    fn update_parameter(op: &mut DefaultOperationRaw) {
+        Json::<T>::update_parameter(op);
+    }
+
+    fn update_response(op: &mut DefaultOperationRaw) {
+        Json::<T>::update_response(op);
+    }
+}
+
 #[cfg(feature = "actix-multipart")]
 impl OperationModifier for actix_multipart::Multipart {
     fn update_parameter(op: &mut DefaultOperationRaw) {
@@ -309,7 +414,9 @@ impl OperationModifier for actix_files::NamedFile {
 macro_rules! impl_param_extractor ({ $ty:ty => $container:ident } => {
     #[cfg(feature = "nightly")]
     impl<T> Apiv2Schema for $ty {
-        default const NAME: Option<&'static str> = None;
+        default fn name() -> Option<String> {
+            None
+        }
 
         default fn raw_schema() -> DefaultSchemaRaw {
             Default::default()
@@ -375,7 +482,9 @@ fn map_schema_to_items(schema: &DefaultSchemaRaw) -> Items {
 /// `formData` can refer to the global definitions.
 #[cfg(feature = "nightly")]
 impl<T: Apiv2Schema> Apiv2Schema for Form<T> {
-    const NAME: Option<&'static str> = T::NAME;
+    fn name() -> Option<String> {
+        T::name()
+    }
 
     fn raw_schema() -> DefaultSchemaRaw {
         T::raw_schema()
@@ -387,8 +496,31 @@ impl_param_extractor!(Query<T> => Query);
 impl_param_extractor!(Form<T> => FormData);
 #[cfg(feature = "serde_qs")]
 impl_param_extractor!(QsQuery<T> => Query);
+#[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+impl_param_extractor!(ValidatedPath<T> => Path);
+#[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+impl_param_extractor!(ValidatedQuery<T> => Query);
+#[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+impl_param_extractor!(ValidatedQsQuery<T> => Query);
 
 macro_rules! impl_path_tuple ({ $($ty:ident),+ } => {
+    #[cfg(all(any(feature = "actix4-validator", feature = "actix3-validator"), feature = "nightly"))]
+    impl<$($ty,)+> Apiv2Schema for ValidatedPath<($($ty,)+)> {}
+
+    #[cfg(all(not(feature = "nightly"), any(feature = "actix4-validator", feature = "actix3-validator")))]
+    impl<$($ty: Apiv2Schema,)+> Apiv2Schema for ValidatedPath<($($ty,)+)> {}
+
+    #[cfg(any(feature = "actix4-validator", feature = "actix3-validator"))]
+    impl<$($ty,)+> OperationModifier for ValidatedPath<($($ty,)+)>
+        where $($ty: Apiv2Schema,)+
+    {
+        fn update_parameter(op: &mut DefaultOperationRaw) {
+            $(
+                Path::<$ty>::update_parameter(op);
+            )+
+        }
+    }
+
     #[cfg(feature = "nightly")]
     impl<$($ty,)+> Apiv2Schema for Path<($($ty,)+)> {}
 
@@ -439,13 +571,23 @@ impl_path_tuple!(A, B);
 impl_path_tuple!(A, B, C);
 impl_path_tuple!(A, B, C, D);
 impl_path_tuple!(A, B, C, D, E);
+impl_path_tuple!(A, B, C, D, E, F);
+impl_path_tuple!(A, B, C, D, E, F, G);
+impl_path_tuple!(A, B, C, D, E, F, G, H);
+impl_path_tuple!(A, B, C, D, E, F, G, H, I);
+impl_path_tuple!(A, B, C, D, E, F, G, H, I, J);
+impl_path_tuple!(A, B, C, D, E, F, G, H, I, J, K);
+impl_path_tuple!(A, B, C, D, E, F, G, H, I, J, K, L);
+impl_path_tuple!(A, B, C, D, E, F, G, H, I, J, K, L, M);
 
 /// Wrapper for wrapping over `impl Responder` thingies (to avoid breakage).
 pub struct ResponderWrapper<T>(pub T);
 
 #[cfg(feature = "nightly")]
 impl<T: Responder> Apiv2Schema for ResponderWrapper<T> {
-    default const NAME: Option<&'static str> = None;
+    default fn name() -> Option<String> {
+        None
+    }
 
     default fn raw_schema() -> DefaultSchemaRaw {
         DefaultSchemaRaw::default()
@@ -458,6 +600,23 @@ impl<T: Responder> Apiv2Schema for ResponderWrapper<T> {}
 #[cfg(not(feature = "nightly"))]
 impl<T: Responder> OperationModifier for ResponderWrapper<T> {}
 
+#[cfg(feature = "actix4")]
+impl Apiv2Schema for actix_web::dev::Response<actix_web::body::BoxBody> {}
+
+#[cfg(feature = "actix4")]
+impl OperationModifier for actix_web::dev::Response<actix_web::body::BoxBody> {}
+
+#[cfg(feature = "actix4")]
+impl<T: Responder> Responder for ResponderWrapper<T> {
+    type Body = T::Body;
+
+    #[inline]
+    fn respond_to(self, req: &HttpRequest) -> HttpResponse<Self::Body> {
+        self.0.respond_to(req)
+    }
+}
+
+#[cfg(not(feature = "actix4"))]
 impl<T: Responder> Responder for ResponderWrapper<T> {
     type Error = T::Error;
     type Future = T::Future;
@@ -474,6 +633,17 @@ impl<T: Responder> Responder for ResponderWrapper<T> {
 #[pin_project]
 pub struct ResponseWrapper<T, H>(#[pin] pub T, pub H);
 
+#[cfg(feature = "actix4")]
+impl<T: Responder, H> Responder for ResponseWrapper<T, H> {
+    type Body = T::Body;
+
+    #[inline]
+    fn respond_to(self, req: &HttpRequest) -> HttpResponse<Self::Body> {
+        self.0.respond_to(req)
+    }
+}
+
+#[cfg(not(feature = "actix4"))]
 impl<T: Responder, H> Responder for ResponseWrapper<T, H> {
     type Error = <T as Responder>::Error;
     type Future = <T as Responder>::Future;
@@ -545,10 +715,10 @@ fn update_security<T>(op: &mut DefaultOperationRaw)
 where
     T: Apiv2Schema,
 {
-    if let (Some(name), Some(scheme)) = (T::NAME, T::security_scheme()) {
+    if let (Some(name), Some(scheme)) = (T::name(), T::security_scheme()) {
         let mut security_map = BTreeMap::new();
         let scopes = scheme.scopes.keys().map(String::clone).collect();
-        security_map.insert(name.into(), scopes);
+        security_map.insert(name, scopes);
         op.security.push(security_map);
     }
 }
@@ -558,8 +728,8 @@ fn update_security_definitions<T>(map: &mut BTreeMap<String, SecurityScheme>)
 where
     T: Apiv2Schema,
 {
-    if let (Some(name), Some(new)) = (T::NAME, T::security_scheme()) {
-        new.update_definitions(name, map);
+    if let (Some(name), Some(new)) = (T::name(), T::security_scheme()) {
+        new.update_definitions(&name, map);
     }
 }
 
@@ -587,6 +757,27 @@ macro_rules! json_with_status {
             }
         }
 
+        #[cfg(feature = "actix4")]
+        impl<T> Responder for $name<T>
+        where
+            T: Serialize + Apiv2Schema,
+        {
+            type Body = BoxBody;
+
+            fn respond_to(self, _: &HttpRequest) -> HttpResponse<BoxBody> {
+                let status: StatusCode = $status;
+                let body = match serde_json::to_string(&self.0) {
+                    Ok(body) => body,
+                    Err(e) => return e.error_response(),
+                };
+
+                HttpResponse::build(status)
+                    .content_type("application/json")
+                    .body(body)
+            }
+        }
+
+        #[cfg(not(feature = "actix4"))]
         impl<T> Responder for $name<T>
         where
             T: Serialize + Apiv2Schema,
@@ -611,7 +802,9 @@ macro_rules! json_with_status {
         where
             T: Serialize + Apiv2Schema,
         {
-            const NAME: Option<&'static str> = T::NAME;
+            fn name() -> Option<String> {
+                T::name()
+            }
 
             fn raw_schema() -> DefaultSchemaRaw {
                 T::raw_schema()
@@ -653,6 +846,18 @@ impl fmt::Display for NoContent {
     }
 }
 
+#[cfg(feature = "actix4")]
+impl Responder for NoContent {
+    type Body = BoxBody;
+
+    fn respond_to(self, _: &HttpRequest) -> HttpResponse<BoxBody> {
+        HttpResponse::build(StatusCode::NO_CONTENT)
+            .content_type("application/json")
+            .finish()
+    }
+}
+
+#[cfg(not(feature = "actix4"))]
 impl Responder for NoContent {
     type Error = Error;
     type Future = Ready<Result<HttpResponse, Error>>;
